@@ -6,9 +6,9 @@ use warnings;
 
 use vars qw /$VERSION/;
 
-$VERSION = '0.01';
+$VERSION = '0.03';
 
-__PACKAGE__->load_classes(qw/ Symbol Quote /);
+__PACKAGE__->load_classes(qw/ Symbol Quote FQMarket /);
 
 =head1 METHODS
 
